"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const cli_ux_1 = require("cli-ux");
const fs = require("fs");
const path = require("path");
const SymbolTree = require("symbol-tree");
const base_1 = require("../base");
const initCalendar_1 = require("../utils/velocity/initCalendar");
const insertClosed_1 = require("../utils/velocity/insertClosed");
const insertDailyVelocity_1 = require("../utils/velocity/insertDailyVelocity");
const insertWeeklyVelocity_1 = require("../utils/velocity/insertWeeklyVelocity");
const fetchChildren_1 = require("../utils/data/fetchChildren");
const fetchCompleted_1 = require("../utils/data/fetchCompleted");
const fetchInitiatives_1 = require("../utils/data/fetchInitiatives");
const crunchRoadmap_1 = require("../utils/initiatives/crunchRoadmap");
const getEmptyCalendar_1 = require("../utils/initiatives/getEmptyCalendar");
const getEmptyRoadmap_1 = require("../utils/initiatives/getEmptyRoadmap");
const prepareInitiativesData_1 = require("../utils/initiatives/prepareInitiativesData");
const prepareOrphanIssues_1 = require("../utils/initiatives/prepareOrphanIssues");
const treeUtils_1 = require("../utils/misc/treeUtils");
const optimizePayload_1 = require("../utils/initiatives/optimizePayload");
class Roadmap extends base_1.default {
    async run() {
        const { flags } = this.parse(Roadmap);
        const userConfig = this.userConfig;
        const cacheDir = this.config.configDir + '/cache/';
        // Creates an array of all closed issues across all teams
        // this is needed to identify which issues have been closed but are not assigned to an initiative
        // This also filters out to only issues listed in the roadmap.teams section, to avoid catch-all rules and issues from being counted twice
        let closedIssues = [];
        const velocityTeamCache = [];
        for (const team of userConfig.teams.filter((t) => userConfig.roadmap.teams.find((teamName) => t.name === teamName) !== undefined)) {
            const teamIssues = await fetchCompleted_1.default(userConfig, this.config.configDir + '/cache/', team.name, flags.date);
            // Merge the two arrays but filter out tickets already present (i.e that could be obtained by different queries)
            closedIssues = [
                ...closedIssues,
                ...teamIssues.filter((tIssue) => closedIssues.find((cIssue) => cIssue.key === tIssue.key) ===
                    undefined),
            ];
            const emptyCalendar = initCalendar_1.default(team.jqlHistory, flags.date);
            const calendarWithClosed = await insertClosed_1.default(emptyCalendar, closedIssues);
            const calendarVelocity = {
                days: insertDailyVelocity_1.default(calendarWithClosed),
                weeks: insertWeeklyVelocity_1.default(calendarWithClosed),
            };
            velocityTeamCache.push(calendarVelocity);
        }
        this.log('Number of issues closed in the period: ' + closedIssues.length);
        // Build an empty weekly calendar, from the first closed issue to the last closed issue
        const emptyCalendar = getEmptyCalendar_1.default(closedIssues);
        // Build velocity metrics per team
        // const velocityTeamCache = await teamVelocityFromCache(userConfig, cacheDir);
        this.log('Fetched velocity metrics for ' + velocityTeamCache.length + ' teams');
        // Fetch all initiatives and its children
        const initiativesIssues = await fetchInitiatives_1.default(userConfig, cacheDir, true, flags.date);
        this.log('Number of initiatives fetched: ' + initiativesIssues.length);
        // Fetch all of the initiatives children and build a tree
        const issuesTree = new SymbolTree();
        const treeRoot = {};
        for (const initiative of initiativesIssues) {
            issuesTree.appendChild(treeRoot, initiative);
            const children = await fetchChildren_1.default(userConfig, initiative.key, cacheDir, true, flags.date);
            for (const l1child of children.filter((ic) => ic.fields[userConfig.jira.fields.parentInitiative] === initiative.key)) {
                issuesTree.appendChild(initiative, l1child);
                for (const l2child of children.filter((ic) => ic.fields[userConfig.jira.fields.parentEpic] === l1child.key)) {
                    issuesTree.appendChild(l1child, l2child);
                }
            }
        }
        cli_ux_1.default.action.start('Preparing metrics for all initiatives in the tree ');
        // Update the tree with completion and week by week metrics
        prepareInitiativesData_1.default(issuesTree, treeRoot, 0, closedIssues, emptyCalendar, userConfig);
        cli_ux_1.default.action.stop(' done');
        cli_ux_1.default.action.start('Creating an export payload');
        const rawInitiativesTree = treeUtils_1.exportTree(issuesTree, treeRoot);
        cli_ux_1.default.action.stop(' done');
        cli_ux_1.default.action.start('Preparing orphan issues');
        // Get an array of all orphaned issues
        const orphanIssues = prepareOrphanIssues_1.default(issuesTree, treeRoot, closedIssues);
        // Add orphaned issues to a weekly calendar
        const orphanWeekly = emptyCalendar.map((week) => {
            const weekIssues = orphanIssues.filter((issue) => week.weekStart === issue.weekStart);
            return Object.assign(Object.assign({}, week), { list: weekIssues, issues: {
                    count: weekIssues.length,
                }, points: {
                    count: weekIssues
                        .map((issue) => issue.points)
                        .reduce((acc, points) => acc + points, 0),
                } });
        });
        cli_ux_1.default.action.stop(' done');
        cli_ux_1.default.action.start('Forecasting future completion');
        const emptyRoadmap = getEmptyRoadmap_1.default(emptyCalendar[emptyCalendar.length - 1], 
        // tslint:disable-next-line: strict-type-predicates
        userConfig.roadmap.forecastWeeks === undefined
            ? 26
            : userConfig.roadmap.forecastWeeks);
        const futureCompletion = crunchRoadmap_1.default(emptyRoadmap, velocityTeamCache, rawInitiativesTree);
        cli_ux_1.default.action.stop(' done');
        const rawInitiativesArtifact = {
            updatedAt: new Date(flags.date).toJSON(),
            host: userConfig.jira.host,
            calendar: {
                completed: emptyCalendar,
                roadmap: emptyRoadmap,
            },
            initiatives: rawInitiativesTree,
            orphanIssues: orphanWeekly,
            futureCompletion: futureCompletion,
        };
        cli_ux_1.default.action.start('Optimizing payload for frontend');
        const slimInitiativesArtifact = optimizePayload_1.default(rawInitiativesArtifact);
        cli_ux_1.default.action.stop(' done');
        // Save archive based on run date
        const initiativesSlimFileStreamArchive = fs.createWriteStream(path.join(cacheDir, 'initiatives-artifacts-' + flags.date + '.json'), { flags: 'w' });
        initiativesSlimFileStreamArchive.write(JSON.stringify(slimInitiativesArtifact));
        initiativesSlimFileStreamArchive.end();
    }
}
exports.default = Roadmap;
Roadmap.description = 'Re-build historical data for a roadmap';
Roadmap.flags = Object.assign(Object.assign({}, base_1.default.flags), { help: command_1.flags.help({ char: 'h' }), type: command_1.flags.string({
        char: 't',
        description: 'Use issues of points for metrics',
        options: ['issues', 'points'],
        default: 'points',
    }), date: command_1.flags.string({
        char: 'd',
        description: 'Set the date to rebuild the history with',
        default: '2020-01-13',
    }) });
