"use strict";
/* eslint max-depth: ["error", 7] */
/* eslint-env es6 */
Object.defineProperty(exports, "__esModule", { value: true });
const cli_ux_1 = require("cli-ux");
const fs = require("fs");
const path = require("path");
const fsNdjson = require("fs-ndjson");
const searchIssues_1 = require("../jira/searchIssues");
const dateUtils_1 = require("../misc/dateUtils");
const date_fns_1 = require("date-fns");
const teamUtils_1 = require("../misc/teamUtils");
const jiraUtils_1 = require("../misc/jiraUtils");
/*
    Fetches all completed issues, per day from a team
*/
const fetchCompleted = async (config, cacheDir, teamName, toDate) => {
    let issues = [];
    console.log('Fetching data for team: ' + teamName);
    const teamConfig = config.teams.find(t => t.name === teamName);
    if (teamConfig !== undefined) {
        const excludeDays = teamConfig.excludeDays === undefined
            ? config.jira.excludeDays
            : [...config.jira.excludeDays, ...teamConfig.excludeDays];
        let toDay = new Date();
        toDay.setDate(toDay.getDate() - 1);
        if (toDate !== undefined) {
            toDay = new Date(toDate);
        }
        const dates = dateUtils_1.getDaysBetweenDates(dateUtils_1.formatDate(teamConfig.jqlHistory), toDay);
        for (const scanDay of dates) {
            if (excludeDays.find((day) => day === scanDay.date.toJSON().slice(0, 10)) === undefined) {
                const issuesDayFilepath = path.join(cacheDir, 'completed-' +
                    teamUtils_1.getTeamId(teamName) +
                    '-' +
                    scanDay.date.toJSON().slice(0, 10) +
                    '.ndjson');
                // If a file is marked clear, it gets deleted
                // .clear file are simply used by the UI to know how many files are pending data refresh
                // And therefore avoid having a very large number of deletion requests triggered at once from the UI
                if (fs.existsSync(issuesDayFilepath + '.clear')) {
                    fs.unlinkSync(issuesDayFilepath + '.clear');
                }
                // eslint-disable-next-line no-negated-condition
                if (!fs.existsSync(issuesDayFilepath)) {
                    cli_ux_1.default.action.start('Fetching data for day: ' +
                        scanDay.date.toJSON().slice(0, 10) +
                        ' to file: ' +
                        issuesDayFilepath);
                    const jqlQuery = teamConfig.jqlCompletion +
                        ' ON(' +
                        scanDay.date.toJSON().slice(0, 10) +
                        ')';
                    const issuesJira = await searchIssues_1.default(config.jira, jqlQuery, 'labels,created,summary,status,issuetype,assignee,' +
                        config.jira.fields.points +
                        ',' +
                        config.jira.fields.originalPoints);
                    // Note: We'd still write an empty file to cache to record the fact that no issues were completed that day
                    const issueFileStream = fs.createWriteStream(issuesDayFilepath, {
                        flags: 'a',
                    });
                    if (issuesJira.length > 0) {
                        for (const issue of issuesJira) {
                            // Adding a closedAt object to record the date at which the issue was actually closed
                            // Attaching points directly to the issues object to avoid having to bring jira-field config specific elements to the UI
                            const openedForDays = date_fns_1.differenceInBusinessDays(new Date(scanDay.date), new Date(issue.fields.created));
                            const updatedIssue = Object.assign(Object.assign({}, issue), { closedAt: scanDay.date.toJSON().slice(0, 10), openedForBusinessDays: openedForDays < 0 ? 0 : openedForDays, weekStart: scanDay.weekStart, weekEnd: scanDay.weekEnd, weekTxt: scanDay.weekTxt, team: teamName, 
                                //              host: config.jira.host,
                                points: jiraUtils_1.returnTicketsPoints(issue, config) });
                            // In some occurences, an issue might already be present (if it was re-open for example)
                            // We filter out any occurence of this issue already be present
                            // This way we only keep the most recent time a ticket is closed and we avoid duplicates.
                            issues = issues.filter((i) => i.key !== updatedIssue.key);
                            issues.push(updatedIssue);
                            issueFileStream.write(JSON.stringify(updatedIssue) + '\n');
                        }
                    }
                    issueFileStream.end();
                    cli_ux_1.default.action.stop(' done');
                }
                else {
                    const cachedIssues = fsNdjson.readFileSync(issuesDayFilepath);
                    for (const issue of cachedIssues) {
                        // In some occurences, an issue might already be present (if it was re-open for example)
                        // We filter out any occurence of this issue already be present
                        // This way we only keep the most recent time a ticket is closed and we avoid duplicates.
                        issues = issues.filter((i) => i.key !== issue.key);
                        issues.push(issue);
                    }
                }
            }
            else {
                cli_ux_1.default.log('Not fetching daily completion for day: ' +
                    scanDay.date.toJSON().slice(0, 10) +
                    ' as per configuration');
            }
        }
    }
    return issues;
};
exports.default = fetchCompleted;
